/****************************************************************************/
/**                                                                        **/
/**                              _pp2bmp.c                                 **/
/**                                                                        **/
/** Convert PowerPaint data to bitmap data                                 **/
/**                                                                        **/
/** Copyright (C) Marcel de Kogel 1997                                     **/
/**     You are not allowed to distribute this software commercially       **/
/**     Please, notify me, if you make any changes to this file            **/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char byte;

/****************************************************************************/
/* Read 8x8 character and put it in the bitmap buffer                       */
/****************************************************************************/
static void convert_cell (byte *pattern,byte *colour,byte *bmp,int width)
{
 int i,fg,bg,k;
 for (i=0;i<8;++i,pattern++,colour++,bmp+=width/2)
 {
  k=pattern[0];
  bg=colour[0]&0x0f;
  fg=(colour[0]>>4)&0x0f;
  if (k&0x80) bmp[0]=(byte)(fg<<4); else bmp[0]=(byte)(bg<<4);
  if (k&0x40) bmp[0]|=(byte)(fg); else bmp[0]|=(byte)(bg);
  if (k&0x20) bmp[1]=(byte)(fg<<4); else bmp[1]=(byte)(bg<<4);
  if (k&0x10) bmp[1]|=(byte)(fg); else bmp[1]|=(byte)(bg);
  if (k&0x08) bmp[2]=(byte)(fg<<4); else bmp[2]=(byte)(bg<<4);
  if (k&0x04) bmp[2]|=(byte)(fg); else bmp[2]|=(byte)(bg);
  if (k&0x02) bmp[3]=(byte)(fg<<4); else bmp[3]=(byte)(bg<<4);
  if (k&0x01) bmp[3]|=(byte)(fg); else bmp[3]|=(byte)(bg);
 }
}

/****************************************************************************/
/* Read a set 8x8 character and put them in the bitmap buffer. Skip first   */
/* two characters of every row, as these do not contain image data.         */
/****************************************************************************/
static void convert_cells (byte *pattern,byte *colour,byte *bmp,int width)
{
 int x,y;
 for (y=0;y<20;++y)
  for (x=2;x<32;++x)
   convert_cell (pattern+x*8+y*256,colour+x*8+y*256,
                 bmp+(x-2)*4+y*width*4,width);
}

/****************************************************************************/
/* Convert PowerPaint data                                                  */
/****************************************************************************/
int pp2bmp (byte *pp,int width,int height,byte *bmp,byte *palette)
{
 byte *pattern,*colour;
 int i,len;
 if (width==240 && height==160)     /* get colour and pattern table offsets */
 { colour=pp+5120; pattern=pp; }
 else
 { colour=pp; pattern=pp+5120; }
 len=(width==240)? 1:2;
 if (height==320) len*=2;
 else if (height==640) len*=4;
 for (i=0;i<len;++i)                /* convert all parts of the image       */
  convert_cells (pattern+i*10240,colour+i*10240,
                 bmp+(i&1)*120+(i/2)*240*160,width);
 return 1;
}
